unit rechnungUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, LCLIntf, LCLType,
  ShellApi,
  FileUtil, LazFileUtils,
  IBConnection, DB, SQLDB,
  LR_Class, LR_Desgn, LR_DBSet, lr_e_pdf,
  PrintersDlgs, Printers,
  mimepart, mimemess, synachar,                         // fürs eMail erstellen/versenden
  DOM, XMLWrite;

type

  { TfrmRechnung }

  TfrmRechnung = class(TForm)
    btnVorschau: TButton;
    btnDruck: TButton;
    btnBearbeiten: TButton;
    btnPDFExport: TButton;
    btnXMLExport: TButton;
    btnGSBatch: TButton;
    btnOutlook_TB: TButton;

    frDBDataSet: TfrDBDataSet;
    frDesigner: TfrDesigner;
    frReport: TfrReport;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    PrintDialog: TPrintDialog;

    IBConnection: TIBConnection;
    SQLTransaction: TSQLTransaction;

    SQLFirma: TSQLQuery;
    SQLFirmaAP: TStringField;
    SQLFirmaBANK: TStringField;
    SQLFirmaBIC: TStringField;
    SQLFirmaBUNDESLAND: TStringField;
    SQLFirmaEMAIL: TStringField;
    SQLFirmaIBAN: TStringField;
    SQLFirmaKONTOINHABER: TStringField;
    SQLFirmaLANDKZ: TStringField;
    SQLFirmaNAME: TStringField;
    SQLFirmaORT: TStringField;
    SQLFirmaPLZ: TStringField;
    SQLFirmaSTRASSE: TStringField;
    SQLFirmaTELEFON: TStringField;
    SQLFirmaUSTIDNR: TStringField;

    SQLRechnungsdruck: TSQLQuery;
    SQLRechnungsdruckBESTELLNR: TStringField;
    SQLRechnungsdruckBEZ: TStringField;
    SQLRechnungsdruckDATUM: TDateField;
    SQLRechnungsdruckDATUMBIS: TDateField;
    SQLRechnungsdruckDATUMVON: TDateField;
    SQLRechnungsdruckEPREIS: TBCDField;
    SQLRechnungsdruckID: TLongintField;
    SQLRechnungsdruckKUNDENID: TLongintField;
    SQLRechnungsdruckKURZBEZ: TStringField;
    SQLRechnungsdruckLEITWEG: TStringField;
    SQLRechnungsdruckMENGE: TBCDField;
    SQLRechnungsdruckMWSTPROZ: TBCDField;
    SQLRechnungsdruckNAME: TStringField;
    SQLRechnungsdruckORT: TStringField;
    SQLRechnungsdruckPLZ: TStringField;
    SQLRechnungsdruckPOS: TLongintField;
    SQLRechnungsdruckSTRASSE: TStringField;
    SQLRechnungsdruckZAHLUNGSZIEL: TLongintField;

    procedure btnVorschauClick(Sender: TObject);
    procedure btnDruckClick(Sender: TObject);
    procedure btnBearbeitenClick(Sender: TObject);
    procedure btnPDFExportClick(Sender: TObject);
    procedure btnXMLExportClick(Sender: TObject);
    procedure btnGSBatchClick(Sender: TObject);
    procedure btnOutlook_TBClick(Sender: TObject);
  private

  public

  end;

var
  frmRechnung: TfrmRechnung;

implementation

{$R *.lfm}

{ TfrmRechnung }


// Hilfsfunktionen/-prozeduren für den ZUGFeRD-Export
function getMwSt(wert, MwStProz: Currency): Currency;
begin
  wert := wert * MwStProz;
  wert := trunc(wert + 0.5); // korrektes runden
  Result := wert / 100;
end;

function waehrung(wert: Double): String;
begin
  str(wert:0:2, Result);
end;

function punktZahl(s: String): String;
var i: integer;
begin
  i := pos(',', s);
  if i > 0 then
    s[i] := '.';
  Result := s;
end;

function EscapeUmlaute(const s: String): String;
begin
  Result := StringReplace(s,'ä','&auml;',[rfReplaceAll]);
  Result := StringReplace(Result,'ö','&ouml;',[rfReplaceAll]);
  Result := StringReplace(Result,'ü','&uuml;',[rfReplaceAll]);
  Result := StringReplace(Result,'Ä','&Auml;',[rfReplaceAll]);
  Result := StringReplace(Result,'Ö','&Ouml;',[rfReplaceAll]);
  Result := StringReplace(Result,'ß','&szlig;',[rfReplaceAll]);
end;

procedure createNode(xmlDoc: TXMLDocument; rootNode: TDOMNode; Element, text, AttrElement, AttrText: String);
var parentNode, textNode: TDOMNode;
begin
  parentNode := xmlDoc.CreateElement(Element);
  textNode := xmlDoc.CreateTextNode(text);
  parentNode.Appendchild(textNode);
  TDOMElement(parentNode).SetAttribute(AttrElement, AttrText);

  rootNode.Appendchild(parentNode);
end;

procedure createNode(xmlDoc: TXMLDocument; rootNode: TDOMNode; Element, text: String);
var parentNode, textNode: TDOMNode;
begin
  parentNode := xmlDoc.CreateElement(Element);
  textNode := xmlDoc.CreateTextNode(text);
  parentNode.Appendchild(textNode);

  rootNode.Appendchild(parentNode);
end;

procedure ZUGFerdExport;
var xmlDoc: TXMLDocument;
    rootNode, parentNode, subNode, sub2Node, sub3Node, sub4Node: TDOMNode;
    sumNetto, sumMwSt: Double;
begin
  with frmRechnung do  // wg. SQLRechnungsdruck
  begin
    sumNetto := 0;
    sumMwSt := 0;
    SQLRechnungsdruck.First;
    while not SQLRechnungsdruck.Eof do
    begin
      sumNetto := sumNetto + SQLRechnungsdruck.FieldByName('Menge').AsFloat * SQLRechnungsdruck.FieldByName('ePreis').AsFloat;
      SQLRechnungsdruck.Next;
	  end;
	  { #todo : Achtung: bei wechselnden MwSt-Sätzen muß hier umgerechnet werden! }
    sumMwSt := getMwSt(sumNetto, SQLRechnungsdruck.FieldByName('MwStProz').AsFloat);

    xmlDoc := TXMLDocument.Create;

    // Header
    rootNode := xmlDoc.CreateElement('rsm:CrossIndustryInvoice');
    TDOMElement(rootNode).SetAttribute('xmlns:rsm', 'urn:un:unece:uncefact:data:standard:CrossIndustryInvoice:100');
    TDOMElement(rootNode).SetAttribute('xmlns:qdt', 'urn:un:unece:uncefact:data:standard:QualifiedDataType:100');
    TDOMElement(rootNode).SetAttribute('xmlns:ram', 'urn:un:unece:uncefact:data:standard:ReusableAggregateBusinessInformationEntity:100');
    TDOMElement(rootNode).SetAttribute('xmlns:xs', 'http://www.w3.org/2001/XMLSchema');
    TDOMElement(rootNode).SetAttribute('xmlns:udt', 'urn:un:unece:uncefact:data:standard:UnqualifiedDataType:100');
    xmlDoc.Appendchild(RootNode);

    parentNode := xmlDoc.CreateElement('rsm:ExchangedDocumentContext');
      subNode := xmlDoc.CreateElement('ram:GuidelineSpecifiedDocumentContextParameter');
        createNode(xmlDoc, subNode, 'ram:ID', 'urn:cen.eu:en16931:2017');
      parentNode.Appendchild(subNode);
    rootNode.Appendchild(parentNode);

    // RechnungsNummer, -Datum, -Fußtext
    parentNode := xmlDoc.CreateComment(' Rechnungskopf ');
    rootNode.Appendchild(parentNode);
    parentNode := xmlDoc.CreateElement('rsm:ExchangedDocument');
      createNode(xmlDoc, parentNode, 'ram:ID', SQLRechnungsdruck.FieldByName('id').AsString);
      createNode(xmlDoc, parentNode, 'ram:TypeCode', '380');  // 380: Commercial invoice
      subNode := xmlDoc.CreateElement('ram:IssueDateTime');
        createNode(xmlDoc, subNode, 'udt:DateTimeString', FormatDateTime('yyyymmdd', SQLRechnungsdruck.FieldByName('Datum').AsDateTime), 'format', '102');
      parentNode.Appendchild(subNode);
      //subNode := xmlDoc.CreateElement('ram:IncludedNote');
      //  createNode(xmlDoc, subNode, 'ram:Content', SQLRechnungsdruck.FieldByName('FussText').AsString));
      //parentNode.Appendchild(subNode);
    rootNode.Appendchild(parentNode);

    // Rechnungspositionen
    parentNode := xmlDoc.CreateElement('rsm:SupplyChainTradeTransaction');

    subNode := xmlDoc.CreateComment(' Rechnungspositionen ');
    parentNode.Appendchild(subNode);
    SQLRechnungsdruck.First;
    while not SQLRechnungsdruck.Eof do
    begin
      subNode := xmlDoc.CreateElement('ram:IncludedSupplyChainTradeLineItem');
        sub2Node := xmlDoc.CreateElement('ram:AssociatedDocumentLineDocument');
        createNode(xmlDoc, sub2Node, 'ram:LineID', SQLRechnungsdruck.FieldByName('Pos').AsString);
      subNode.Appendchild(sub2Node);
      sub2Node := xmlDoc.CreateElement('ram:SpecifiedTradeProduct');
        //createNode(xmlDoc, sub2Node, 'ram:GlobalID schemeID="0160"', SQLRechnungsdruck.FieldByName('EAN').AsString); // 0160 - GS1 Global Trade Item Number (GTIN, EAN)
        //createNode(xmlDoc, sub2Node, 'ram:SellerAssignedID', SQLRechnungsdruck.FieldByName('ArtikelNr').AsString);
        createNode(xmlDoc, sub2Node, 'ram:Name', SQLRechnungsdruck.FieldByName('KurzBez').AsString);
        createNode(xmlDoc, sub2Node, 'ram:Description', SQLRechnungsdruck.FieldByName('Bez').AsString);
      subNode.Appendchild(sub2Node);

      // Einzelpreis; dieser muß einmal Brutto, einmal Netto ausgewiesen werden
      // wobei Brutto nur heißt: ohne Nachlässe/Rabatt; beide Preise sind ohne MwSt
      sub2Node := xmlDoc.CreateElement('ram:SpecifiedLineTradeAgreement');
        sub3Node := xmlDoc.CreateElement('ram:GrossPriceProductTradePrice');
          createNode(xmlDoc, sub3Node, 'ram:ChargeAmount', waehrung(SQLRechnungsdruck.FieldByName('ePreis').AsFloat));
        sub2Node.Appendchild(sub3Node);
        sub3Node := xmlDoc.CreateElement('ram:NetPriceProductTradePrice');
          createNode(xmlDoc, sub3Node, 'ram:ChargeAmount', waehrung(SQLRechnungsdruck.FieldByName('ePreis').AsFloat));
        sub2Node.Appendchild(sub3Node);
      subNode.Appendchild(sub2Node);

      // Menge
      sub2Node := xmlDoc.CreateElement('ram:SpecifiedLineTradeDelivery');
        createNode(xmlDoc, sub2Node, 'ram:BilledQuantity', punktZahl(SQLRechnungsdruck.FieldByName('Menge').AsString), 'unitCode', 'H87');  // H87: Preis per Stück
      subNode.Appendchild(sub2Node);

      // MwSt
      sub2Node := xmlDoc.CreateElement('ram:SpecifiedLineTradeSettlement');
        sub3Node := xmlDoc.CreateElement('ram:ApplicableTradeTax');
        createNode(xmlDoc, sub3Node, 'ram:TypeCode', 'VAT');
        createNode(xmlDoc, sub3Node, 'ram:CategoryCode', 'S'); // S = Umsatzsteuer fällt mit Normalsatz an
        createNode(xmlDoc, sub3Node, 'ram:RateApplicablePercent', punktZahl(SQLRechnungsdruck.FieldByName('MwStProz').AsString));
      sub2Node.Appendchild(sub3Node);
        sub3Node := xmlDoc.CreateElement('ram:SpecifiedTradeSettlementLineMonetarySummation');
        createNode(xmlDoc, sub3Node, 'ram:LineTotalAmount', Waehrung(SQLRechnungsdruck.FieldByName('Menge').AsFloat * SQLRechnungsdruck.FieldByName('ePreis').AsFloat));
      sub2Node.Appendchild(sub3Node);
      subNode.Appendchild(sub2Node);

      parentNode.Appendchild(subNode);

      SQLRechnungsdruck.Next;
	  end;

    subNode := xmlDoc.CreateElement('ram:ApplicableHeaderTradeAgreement');

    // Firmendaten
      sub2Node := xmlDoc.CreateComment(' Firmendaten ');
    subNode.Appendchild(sub2Node);

      sub2Node := xmlDoc.CreateElement('ram:SellerTradeParty');
      createNode(xmlDoc, sub2Node, 'ram:Name', SQLFirma.FieldByName('Name').AsString);
      // Kontaktdaten des Verkäufers
      sub3Node := xmlDoc.CreateElement('ram:DefinedTradeContact');
        createNode(xmlDoc, sub3Node, 'ram:PersonName', SQLFirma.FieldByName('AP').AsString);
          sub4Node := xmlDoc.CreateElement('ram:TelephoneUniversalCommunication');
          createNode(xmlDoc, sub4Node, 'ram:CompleteNumber', SQLFirma.FieldByName('Telefon').AsString);
        sub3Node.Appendchild(sub4Node);
          sub4Node := xmlDoc.CreateElement('ram:EmailURIUniversalCommunication');
          createNode(xmlDoc, sub4Node, 'ram:URIID', SQLFirma.FieldByName('eMail').AsString);
        sub3Node.Appendchild(sub4Node);
      sub2Node.Appendchild(sub3Node);
      // Firmenanschrift
      sub3Node := xmlDoc.CreateElement('ram:PostalTradeAddress');
        createNode(xmlDoc, sub3Node, 'ram:PostcodeCode', SQLFirma.FieldByName('Plz').AsString);
        createNode(xmlDoc, sub3Node, 'ram:LineOne', SQLFirma.FieldByName('Strasse').AsString);
        createNode(xmlDoc, sub3Node, 'ram:CityName', SQLFirma.FieldByName('Ort').AsString);
        createNode(xmlDoc, sub3Node, 'ram:CountryID', SQLFirma.FieldByName('LandKz').AsString);
      sub2Node.Appendchild(sub3Node);
      sub3Node := xmlDoc.CreateElement('ram:SpecifiedTaxRegistration');
        createNode(xmlDoc, sub3Node, 'ram:ID', SQLFirma.FieldByName('UStIDNr').AsString, 'schemeID', 'VA');  // VA - UstIDNr  FC - Steuernummer 123/456/78900
      sub2Node.Appendchild(sub3Node);
    subNode.Appendchild(sub2Node);

    // Kundendaten
      sub2Node := xmlDoc.CreateComment(' Kundendaten ');
    subNode.Appendchild(sub2Node);
      sub2Node := xmlDoc.CreateElement('ram:BuyerTradeParty');
      createNode(xmlDoc, sub2Node, 'ram:ID', SQLRechnungsdruck.FieldByName('KundenID').AsString);
      createNode(xmlDoc, sub2Node, 'ram:Name', SQLRechnungsdruck.FieldByName('Name').AsString);
      sub3Node := xmlDoc.CreateElement('ram:PostalTradeAddress');
        createNode(xmlDoc, sub3Node, 'ram:PostcodeCode', SQLRechnungsdruck.FieldByName('Plz').AsString);
        createNode(xmlDoc, sub3Node, 'ram:LineOne', SQLRechnungsdruck.FieldByName('Strasse').AsString);
        createNode(xmlDoc, sub3Node, 'ram:CityName', SQLRechnungsdruck.FieldByName('Ort').AsString);
        createNode(xmlDoc, sub3Node, 'ram:CountryID', 'DE');
      sub2Node.Appendchild(sub3Node);
    subNode.Appendchild(sub2Node);
    parentNode.Appendchild(subNode);

    // Leistungsdatum  { #todo : Das Leistungsdatum muß noch im Programm festgelegt werden }
    subNode := xmlDoc.CreateElement('ram:ApplicableHeaderTradeDelivery');
      sub2Node := xmlDoc.CreateElement('ram:ActualDeliverySupplyChainEvent');
        sub3Node := xmlDoc.CreateElement('ram:OccurrenceDateTime');
        createNode(xmlDoc, sub3Node, 'udt:DateTimeString', FormatDateTime('yyyymmdd', SQLRechnungsdruck.FieldByName('Datum').AsDateTime), 'format', '102');
      sub2Node.Appendchild(sub3Node);
    subNode.Appendchild(sub2Node);
    parentNode.Appendchild(subNode);

    // Zahlungsdaten
      subNode := xmlDoc.CreateComment(' Zahlungsdaten ');
    parentNode.Appendchild(subNode);

    subNode := xmlDoc.CreateElement('ram:ApplicableHeaderTradeSettlement');
    createNode(xmlDoc, subNode, 'ram:InvoiceCurrencyCode', 'EUR');

    // Überweisung an Bankkonto
    sub2Node := xmlDoc.CreateElement('ram:SpecifiedTradeSettlementPaymentMeans');
      createNode(xmlDoc, sub2Node, 'ram:TypeCode', '30'); // Code 30 (31 ist falsch!): Überweisung durch den Kunden  // siehe https://projekte.kosit.org/xrechnung/xrechnung/-/issues/41
      sub3Node := xmlDoc.CreateElement('ram:PayeePartyCreditorFinancialAccount');
        createNode(xmlDoc, sub3Node, 'ram:IBANID', SQLFirma.FieldByName('IBAN').AsString);
        createNode(xmlDoc, sub3Node, 'ram:AccountName', SQLFirma.FieldByName('KontoInhaber').AsString);
      sub2Node.Appendchild(sub3Node);
      sub3Node := xmlDoc.CreateElement('ram:PayeeSpecifiedCreditorFinancialInstitution');
        createNode(xmlDoc, sub3Node, 'ram:BICID', SQLFirma.FieldByName('BIC').AsString);
      sub2Node.Appendchild(sub3Node);
    subNode.Appendchild(sub2Node);

    // Bei mehreren MwSt-Sätzen muß dieser Bereich mehrfach ausgeführt werden
    sub2Node := xmlDoc.CreateElement('ram:ApplicableTradeTax');
      createNode(xmlDoc, sub2Node, 'ram:CalculatedAmount', waehrung(sumMwSt));
      createNode(xmlDoc, sub2Node, 'ram:TypeCode', 'VAT');
      createNode(xmlDoc, sub2Node, 'ram:BasisAmount', waehrung(sumNetto));
      createNode(xmlDoc, sub2Node, 'ram:CategoryCode', 'S');
      createNode(xmlDoc, sub2Node, 'ram:RateApplicablePercent', SQLRechnungsdruck.FieldByName('MwStProz').AsString);
    subNode.Appendchild(sub2Node);

    // Zahlungsziel
    sub2Node := xmlDoc.CreateElement('ram:SpecifiedTradePaymentTerms');
      createNode(xmlDoc, sub2Node, 'ram:Description', 'Zahlbar bis zum ' + FormatDateTime('dd.mm.yyyy', SQLRechnungsdruck.FieldByName('Datum').AsDateTime + SQLRechnungsdruck.FieldByName('Zahlungsziel').AsInteger) + ' netto Kasse');
    subNode.Appendchild(sub2Node);

    // Zahlungsdaten
    sub2Node := xmlDoc.CreateElement('ram:SpecifiedTradeSettlementHeaderMonetarySummation');
      createNode(xmlDoc, sub2Node, 'ram:LineTotalAmount', waehrung(sumNetto));
      createNode(xmlDoc, sub2Node, 'ram:ChargeTotalAmount', waehrung(0));
      createNode(xmlDoc, sub2Node, 'ram:AllowanceTotalAmount', waehrung(0));
      createNode(xmlDoc, sub2Node, 'ram:TaxBasisTotalAmount', waehrung(sumNetto));
      createNode(xmlDoc, sub2Node, 'ram:TaxTotalAmount', waehrung(sumMwSt), 'currencyID', 'EUR');
      createNode(xmlDoc, sub2Node, 'ram:GrandTotalAmount', waehrung(sumNetto + sumMwSt));
      createNode(xmlDoc, sub2Node, 'ram:TotalPrepaidAmount', waehrung(0));
      createNode(xmlDoc, sub2Node, 'ram:DuePayableAmount', waehrung(sumNetto + sumMwSt));
    subNode.Appendchild(sub2Node);

    parentNode.Appendchild(subNode);
    rootNode.Appendchild(parentNode);

    // xml-Datei speichern
    writeXMLFile(xmlDoc, 'factur-x.xml');

  end;
end;

procedure drucken(Ausgabe: Word);
var i: word;
    FromPage, ToPage, NumberCopies: Integer;
    PrinterIndex: Integer;
    pdfName: String;
    Berichtname: String;
begin
  with frmRechnung do
  begin
    Berichtname := 'Rechnung.lrf';
    frReport.LoadFromFile('Berichte\' + Berichtname);
    SQLRechnungsdruck.First;
    while not SQLRechnungsdruck.EoF do
    begin
      SQLRechnungsdruck.Next;
	  end;
    SQLRechnungsdruck.First;
    if frReport.PrepareReport then
    begin
      case Ausgabe of
        1:  // Drucken
        begin
	        PrinterIndex := Printer.PrinterIndex;
	        FromPage := 1;
	        ToPage := 99;
	        frReport.PrepareReport;
	        PrintDialog.Options := [poPageNums ];
	        PrintDialog.Copies := 1;
	        PrintDialog.Collate := true;
	        PrintDialog.FromPage := 1;
	        PrintDialog.ToPage := frReport.EMFPages.Count;
	        PrintDialog.MaxPage := frReport.EMFPages.Count;
	        if PrintDialog.Execute then
	        begin
	          if (Printer.PrinterIndex <> PrinterIndex)
	          or frReport.CanRebuild                    // ... only makes sense if we can reformat the report
	          or frReport.ChangePrinter(PrinterIndex, Printer.PrinterIndex) then
	            frReport.PrepareReport;
	          if PrintDialog.PrintRange = prPageNums then
	          begin
	            FromPage := PrintDialog.FromPage;
	            ToPage := PrintDialog.ToPage;
	          end;
	          NumberCopies := PrintDialog.Copies;
	          frReport.PrintPreparedReport(IntToStr(FromPage) + '-' + IntToStr(ToPage), NumberCopies);
	        end;
        end;

        2: // Berichtsvorschau
	      begin
          SQLRechnungsdruck.First;
          frReport.ShowPreparedReport;
			  end;

        3: // Bericht bearbeiten (vorher Sicherungskopie erstellen!)
	      begin
          if not FileExists('Berichte\backup') then
            CreateDir('Berichte\backup');
          i := 1;
          while FileExists('Berichte\backup\' + Berichtname + ';' + IntToStr(i)) do
            i := i + 1;
          CopyFile('Berichte\' + Berichtname, 'Berichte\backup\' + Berichtname + ';' + IntToStr(i));
          frReport.DesignReport; // Achtung: der frDesigner muß auf der Form stehen!
			  end;

        4: // als pdf speichern
	      begin
          pdfName := 'Invoice.pdf'; //'Rechnung-' + SQLRechnungsdruck.FieldByName('id').AsString + '.pdf';
	        if FileExists(pdfName) then
	          DeleteFile(pdfName);
	        if FileExists(pdfName) then
	          Application.MessageBox('PDF-Datei kann nicht erstellt werden.'+chr(13)+'Sie ist in einer anderer Anwendung geöffnet.'+CHR(13)+'Bitte Anwendung schließen.', 'Achtung', MB_ICONWARNING)
				  else
	        begin
            if frReport.PrepareReport then
            begin
              // siehe f:\lazarus\components\lazreport\source\addons\pdfexport\readme.txt
              frReport.ExportTo(TFrTNPDFExportFilter, pdfName);
              // frReport.ExportTo(TlrPdfExportFilter, pdfName);  // alter Filter; Format <> A4
            end;
			    end;
			  end;

      end; // case
    end;
	end;
end;

procedure Outlook_TB_Erstellung;
var fileName: String;
    slText, slHTML: TStrings;
    pMix, pRel, pAlt: TMimePart;
    MIMEMessage: TMimeMess;
    FROM, TOEMAIL: String;
    RechnungID: Integer;
begin
  RechnungID := 3197;
  fileName := 'Rechnung_' + IntToStr(RechnungID) + '.pdf';
  CopyFile('invoice.GS.pdf', fileName);

  MIMEMessage := TMimeMess.Create;
  try
    // Hier muß es sich um eine in der Anwendung existierende
    // eMail-Adresse handeln
    FROM := 'info@example.com';
    TOEMAIL := 'info@example.com';

    // Struktur: https://stackoverflow.com/questions/3902455/mail-multipart-alternative-vs-multipart-mixed
    // http://synapse.ararat.cz/doku.php/public:howto:mimeparts
    // https://borland.public.delphi.thirdpartytools.general.narkive.com/Z8dlJttu/using-synapse-to-send-emails#post19
    slHTML := TStringList.Create;
    slHTML.Add('<!DOCTYPE html><html><head><meta http-equiv="content-type" content="text/html; charset=UTF-8"></head>');
    slHTML.Add('<body>');
    slHTML.Add('Sehr geehrte Damen und Herren,<br />');
    slHTML.Add('<br />');
    slHTML.Add('als Anhang die aktuelle Rechnung.<br />');
    slHTML.Add('<br />');
    slHTML.Add('-- <br />');
    slHTML.Add(EscapeUmlaute('Mit freundlichen Grüßen') + '<br />');
    slHTML.Add('PS Bielefeld<br />');
    slHTML.Add('<br />');
    slHTML.Add('<img src="cid:FirmenLogo" border="0">');
    slHTML.Add('</body></html>');

    slText := TStringList.Create;
    slText.Add('(Text)');
    slText.Add('Sehr geehrte Damen und Herren,');
    slText.Add('');
    slText.Add('als Anhang die aktuelle Rechnung.');
    slText.Add('');
    slText.Add('-- ');
    slText.Add('Mit freundlichen Grüßen');
    slText.Add('PS Bielefeld');

    MIMEMessage.Header.Date := Now;
    MIMEMessage.Header.From := FROM;
    MIMEMessage.Header.ToList.Clear;
    MIMEMessage.Header.ToList.Add(TOEMAIL);
    MIMEMessage.Header.Subject := 'Rechnung';
    MIMEMessage.Header.XMailer := 'SoftBrain';
    MIMEMessage.Header.CustomHeaders.Add('X-Unsent: 1');

  	//multiparts (parent-parts)
  	pMix := MIMEMessage.AddPartMultipart('mixed', nil);
  	pRel := MIMEMessage.AddPartMultipart('related', pMix);
  	pAlt := MIMEMessage.AddPartMultipart('alternative', pRel);

    MIMEMessage.AddPartTextEx(slText, pAlt, UTF_8, True, ME_8BIT); // nur bei AddPartText_Ex_ kann der Charset übergeben werden!

    MIMEMessage.AddPartHTML(slHTML, pAlt);

    // HTML-Logo einfügen
    MIMEMessage.AddPartHTMLBinaryFromFile('Logo.png', 'FirmenLogo', pRel);

    // Rechnungsdatei anhängen
    MIMEMessage.AddPartBinaryFromFile(fileName, pMix);

    MIMEMessage.EncodeMessage;
    MIMEMessage.Lines.SaveToFile('OutMail_' + IntToStr(RechnungID) + '.eml');
    OpenDocument('OutMail_' + IntToStr(RechnungID) + '.eml');

  finally
    MIMEMessage.Free;
    slHTML.Free;
    slText.Free;
  end;

end;

procedure TfrmRechnung.btnDruckClick(Sender: TObject);
begin
  drucken(1); // Drucken
end;

procedure TfrmRechnung.btnVorschauClick(Sender: TObject);
begin
  drucken(2); // Vorschau
end;

procedure TfrmRechnung.btnBearbeitenClick(Sender: TObject);
begin
  drucken(3); // DesignReport
end;

procedure TfrmRechnung.btnPDFExportClick(Sender: TObject);
begin
  drucken(4); // PDF-Export
end;

procedure TfrmRechnung.btnXMLExportClick(Sender: TObject);
begin
   ZugFerdExport;
end;

procedure TfrmRechnung.btnGSBatchClick(Sender: TObject);
begin
  // jetzt das Postscript-Script per Batch ausführen!
  ShellExecute(0, PChar('open'), PChar('gs-create.bat'), Nil, Nil, 1);
end;

procedure TfrmRechnung.btnOutlook_TBClick(Sender: TObject);
begin
  Outlook_TB_Erstellung;
end;

end.

